<?php

namespace Drupal\webform_cpf\Element;

use Drupal\Core\Render\Element;
use Drupal\Core\Render\Element\FormElement;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\cpf\CpfService;

/**
 * Provides a 'webform_cpf'.
 *
 * @FormElement("webform_cpf")
 */
class WebformCpf extends FormElement implements ContainerFactoryPluginInterface {

  /**
   * The cpf service.
   *
   * @var \Drupal\cpf\CpfService
   */
  protected $cpf;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('cpf')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, CpfService $cpf) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->cpf = $cpf;
  }

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = get_class($this);
    return [
      '#input' => TRUE,
      '#size' => 60,
      '#process' => [
        [$class, 'processWebformCpf'],
        [$class, 'processAjaxForm'],
      ],
      '#element_validate' => [
        [$this, 'validateWebformCpf'],
      ],
      '#pre_render' => [
        [$class, 'preRenderWebformCpf'],
      ],
      '#theme' => 'input__webform_cpf',
      '#theme_wrappers' => ['form_element'],
    ];
  }

  /**
   * Processes a 'webform_cpf' element.
   */
  public static function processWebformCpf(&$element, FormStateInterface $form_state, &$complete_form) {
    return $element;
  }

  /**
   * Webform element validation handler for #type 'webform_cpf'.
   */
  public function validateWebformCpf(&$element, FormStateInterface $form_state, &$complete_form) {

    $cpf = $this->cpf->digits($element["#value"]);

    if (!$this->cpf->isValid($cpf)) {
      $form_state->setErrorByName($element["#webform_key"],
        t('The CPF number %cpf is not valid', ['%cpf' => $element["#value"]]));
    }
  }

  /**
   * Prepares a #type 'webform-cpf' render element for theme_element().
   *
   * @param array $element
   *   An associative array containing the properties of the element.
   *   Properties used: #title, #value, #description, #size, #maxlength,
   *   #placeholder, #required, #attributes.
   *
   * @return array
   *   The $element with prepared variables ready for theme_element().
   */
  public static function preRenderWebformCpf(array $element) {
    $element['#attributes']['type'] = 'text';
    Element::setAttributes($element, ['id',
      'name',
      'value',
      'size',
      'maxlength',
      'placeholder',
    ]);
    static::setAttributes($element, ['form-text', 'webform-cpf']);
    return $element;
  }

}
